package com.v1.controller.business;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.config.QianZhuConfig;
import com.hai.entity.ApiMerchants;
import com.hai.entity.ApiStarbucksProducts;
import com.hai.model.ResponseData;
import com.hai.order.type.OrderProductType;
import com.hai.service.*;
import com.v1.controller.ThirdPartyController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping(value = "/starbucks")
@Api(value = "星巴克业务")
public class StarbucksController {


    private static Logger log = LoggerFactory.getLogger(ThirdPartyController.class);

    @Resource
    private ApiMerchantsService apiMerchantsService;

    @Resource
    private SecConfigService secConfigService;

    @Resource
    private ApiIpAddressService apiIpAddressService;

    @Resource
    private ApiProductService apiProductService;


    @RequestMapping(value = "/starbucksStore", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "查询星巴克门店")
    public ResponseData starbucksStore(@RequestBody JSONObject object , HttpServletRequest request ) {
        try {

            String ip = "";
            // 有的user可能使用代理，为处理用户使用代理的情况，使用x-forwarded-for
            if  (request.getHeader("x-forwarded-for") == null)  {
                ip = request.getRemoteAddr();
            }  else  {
                ip = request.getHeader("x-forwarded-for");
            }

            if (!apiIpAddressService.validationIpAddressLegal(object.getString("merchId") , ip)) {
                log.error("getRechargeProduct error!", "非法ip地址，请联系管理人员！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.IP_ERROR, "非法ip地址，请联系管理人员！");
            }

            if (StringUtils.isBlank(object.getString("apiKey")) ||
                    StringUtils.isBlank(object.getString("merchId")) ||
                    StringUtils.isBlank(object.getString("lat")) ||
                    StringUtils.isBlank(object.getString("lon")) ||
                    StringUtils.isBlank(object.getString("sign"))
            ) {
                log.error("getRechargeProduct error!", "请求参数校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR);
            }

            ApiMerchants apiMerchants = apiMerchantsService.findByMchId(object.getString("merchId"));

            if (apiMerchants == null) {
                log.error("getRechargeProduct error!", "帐户号错误！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.ACCOUNT_NOT_EXIST);
            }

            if (!secConfigService.isConfig(OrderProductType.PRODUCT_TYPE1.getApiCode(), object.getString("merchId"))) {
                log.error("getAllCities error!", "当前帐户无权限查询！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_NOT_AUTHORIZED, "");
            }

            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , object.getString("apiKey"));
            map.put("merchId" , object.getString("merchId"));
            map.put("lat" , object.getString("lat"));
            map.put("lon" , object.getString("lon"));

            if (!secConfigService.isSignVerify(map , object.getString("sign"))) {
                log.error("getRechargeProduct error!", "签名校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.SIGN_VERIFY);
            }

            JSONObject jsonObject = QianZhuConfig.getStarbucksStore(object.getString("lat") , object.getString("lon"));

            if (jsonObject.getBoolean("success")) {
                return ResponseMsgUtil.success(jsonObject.getString("data"));
            } else {
                log.error("getRechargeProduct error!", "查询失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR , jsonObject.getString("message"));
            }

        } catch (Exception e) {
            log.error("HighOpenApiController --> getRechargeProduct() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getStarbucksProducts", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "查询星巴克商品")
    public ResponseData getStarbucksProducts(@RequestBody JSONObject object , HttpServletRequest request ) {
        try {

            String ip = "";
            // 有的user可能使用代理，为处理用户使用代理的情况，使用x-forwarded-for
            if  (request.getHeader("x-forwarded-for") == null)  {
                ip = request.getRemoteAddr();
            }  else  {
                ip = request.getHeader("x-forwarded-for");
            }

            if (!apiIpAddressService.validationIpAddressLegal(object.getString("merchId") , ip)) {
                log.error("getRechargeProduct error!", "非法ip地址，请联系管理人员！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.IP_ERROR, "非法ip地址，请联系管理人员！");
            }

            if (StringUtils.isBlank(object.getString("apiKey")) ||
                    StringUtils.isBlank(object.getString("merchId")) ||
                    StringUtils.isBlank(object.getString("sign"))
            ) {
                log.error("getRechargeProduct error!", "请求参数校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR);
            }

            ApiMerchants apiMerchants = apiMerchantsService.findByMchId(object.getString("merchId"));

            if (apiMerchants == null) {
                log.error("getRechargeProduct error!", "帐户号错误！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.ACCOUNT_NOT_EXIST);
            }

            if (!secConfigService.isConfig(OrderProductType.PRODUCT_TYPE1.getApiCode(), object.getString("merchId"))) {
                log.error("getAllCities error!", "当前帐户无权限查询！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_NOT_AUTHORIZED, "");
            }

            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , object.getString("apiKey"));
            map.put("merchId" , object.getString("merchId"));

            if (!secConfigService.isSignVerify(map , object.getString("sign"))) {
                log.error("getRechargeProduct error!", "签名校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.SIGN_VERIFY);
            }

            Map<String, Object> mapProduct = new HashMap<>();

            List<ApiStarbucksProducts> list = apiProductService.getStarbucksProducts(mapProduct);

            for (ApiStarbucksProducts starbucksProducts : list) {
                if (starbucksProducts.getMarketGrandePrice().compareTo(new BigDecimal(1000)) > 0) {
                    starbucksProducts.setMarketGrandePrice(null);
                }
                if (starbucksProducts.getMarketTallPrice().compareTo(new BigDecimal(1000)) > 0) {
                    starbucksProducts.setMarketTallPrice(null);
                }
                if (starbucksProducts.getMarketVentiPrice().compareTo(new BigDecimal(1000)) > 0) {
                    starbucksProducts.setMarketVentiPrice(null);
                }
                starbucksProducts.setSalesTallPrice(starbucksProducts.getMarketTallPrice());
                starbucksProducts.setSalesVentiPrice(starbucksProducts.getMarketVentiPrice());
                starbucksProducts.setSalesGrandePrice(starbucksProducts.getMarketGrandePrice());
            }

            return ResponseMsgUtil.success(list);

        } catch (Exception e) {
            log.error("HighOpenApiController --> getRechargeProduct() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
